import pytest
from transmute_core.object_serializers import (
    ListSerializer
)
from typing import List


def test_list_serializer(object_serializer_set):
    x = ["10"]

    obj = object_serializer_set.load([int], x)
    assert obj == [10]
    obj = object_serializer_set.dump([int], obj)
    assert obj == [10]


def test_list_serializer_json_schema(object_serializer_set):
    schema = object_serializer_set.to_json_schema([str])
    assert schema == {
        "type": "array",
        "items": {"type": "string"}
    }


def test_object_serializer_set_can_handle(object_serializer_set):
    schema = object_serializer_set.to_json_schema([str])
    assert schema == {
        "type": "array",
        "items": {"type": "string"}
    }


@pytest.mark.parametrize("cls,inp,out,expected_out", [
    (List[int], [1, 2], [1,2], {"type": "array", "items": {"type": "integer"}})
])
def test_serializer_values(object_serializer_set,
                           cls, inp, out, expected_out):
    assert object_serializer_set.to_json_schema(cls) == expected_out
    assert object_serializer_set.load(cls, out) == inp
    assert object_serializer_set.dump(cls, inp) == out



from ...object_serializers.primitive_serializer import BoolSerializer
import pytest


@pytest.mark.parametrize("object, result", [
    ("true", True), ("false", False),
    ("some_other_str", False), ([], False),
    (["non-empty-list"], True), ({}, False),
    ({"non_empty_dict": "test_val"}, True)
])
def test_primitive_serializer_bool_serializer(object, result):
    serializer = BoolSerializer()
    assert BoolSerializer.load(serializer, object) == result



import pytest
from transmute_core.http_parameters import (
    get_swagger_parameters, ParamSet, Param, Parameters
)
from transmute_core.function.signature import Argument, NoDefault
from datetime import datetime


def test_post_schema_swagger(parameter_post_schema, context):
    assert get_swagger_parameters(parameter_post_schema, context)[0].to_primitive() == {
        "in": "body",
        "name": "body",
        "required": True,
        "description": "",
        "schema": {
            "type": "integer"
        }
    }


def test_header_only_schema(context):
    params = Parameters(
        path=ParamSet({"path": Param(argument_name="path",
                                     arginfo=Argument("path", NoDefault, int))}),
    )
    assert get_swagger_parameters(params, context)[0].to_primitive() == {
        "in": "path",
        "name": "path",
        "required": True,
        "type": "integer",
        "description": "",
    }

@pytest.mark.parametrize("inp, expected", [
    (Parameters(query=ParamSet({
        "query": Param(argument_name="query",
                       arginfo=Argument("query", NoDefault, datetime))
     })),
    {
        "in": "query",
        "name": "query",
        "required": True,
        "type": "string",
        "format": "date-time",
        "collectionFormat": "multi",
        "description": ""
     }
    ),
    (Parameters(path=ParamSet({
        "path": Param(argument_name="path",
                      arginfo=Argument("path", NoDefault, datetime))
     })),
     {
        "in": "path",
        "name": "path",
        "required": True,
        "type": "string",
        "format": "date-time",
        "description": ""
     }
    ),
    (Parameters(header=ParamSet({
        "path": Param(argument_name="path",
                        arginfo=Argument("path", NoDefault, datetime))
     })),
     {
        "in": "header",
        "name": "path",
        "required": True,
        "type": "string",
        "format": "date-time",
        "description": ""
     }
    )
])
def test_additional_type_info(context, inp, expected):
    """
    query parameter should have additional type data
    (like format) added to it.
    """
    assert get_swagger_parameters(inp, context)[0].to_primitive() == expected



import attr
from collections import Mapping

@attr.s
class Response(object):
    result = attr.ib()
    code = attr.ib(default=200)
    headers = attr.ib(
        validator=[attr.validators.instance_of(Mapping)],
        default=attr.Factory(dict)
    )
    success = attr.ib(
        validator=[attr.validators.instance_of(bool)],
        default=True
    )
